#######################################
# Hi/Low Temp/Salinity Model Simulation
#######################################
# Clearing Workspace and loading packages
# rm(list = ls())
# setwd("/Users/hurford1/Desktop/RittenhouseEtAl_Code") # Set this to preferred working directory 
library(PBSddesolve)
library(ggplot2)
library(cowplot)

### Mean Annual temperatures and salinities
SimTemp <- c(6, 6, 10, 10)
SimSal <- c(15, 25, 15, 25)
nsim <- length(SimTemp)

### The Model
## Temperature/Development
# Temperature Seasonality
Temp <- function(t){
  a - b * cos(2 * pi * t / 365)
}

# planktonic delay
GammaP <- function(Temp){
  ((Betap * (Temp - Tref + alphap)) / (alphap)) ^ 2
}

# chalimus delay
GammaC <- function(Temp){
  ((Betac * (Temp - Tref + alphac)) / (alphac)) ^ 2
}

## Salinity/Mortality
# Salinity seasonality
S <- function(t){
  c - d * cos(2 * pi * t / 365)
}

# egg viability fit to data from Johnson & Albright 1991
V <- function(S){
  Ve <- sigma0e + sigma1e * S
  ifelse(Ve < 0, 0, Ve)  
}

# adult mortality based on Connors 2008
mua <- function(S){
  logsurada <- (sigma0a + sigma1a * S)
  suradaest <- exp(logsurada)
  mewa <- 24 / suradaest
  ifelse(mewa > 1, 1, mewa)  
}

# chalimus mortality is similar to adult so we use same eqs
muc <- function(S){
  logsurcha <- (sigma0c + sigma1c * S)
  surchaest <- exp(logsurcha)
  mewc <- 24 / surchaest
  ifelse(mewc > 1, 1, mewc) 
}

# Nauplius mortality from Brooks & Stucchi 2005
mup <- function(S){
  mup <- sigma0p + sigma1p * S
  ifelse(mup < 0, 0, mup)
}

# Copepodid Mortality assumed to be similar to nauplius mortality
mui <- function(S){
  mui <- sigma0i + sigma1i * S
  ifelse(mui < 0, 0, mui)
}

## The DDE system
# Function to calculate the gradient
lice <- function(t, y){
  A <- y[1]
  P <- y[2]
  tp <- y[3]
  I <- y[4]
  C <- y[5]
  tc <- y[6]
  PhiP <- y[7]
  PhiC <- y[8]
  
  # Apply timelags only once enough history has been established:
  
  if (t - tp < t0) {
    # The value of the lagged variables. Setting them equal to 0 means no lag.
    lagp <- y * 0
  } else {
    # if the logical test passses we can calculate the lags
    lagp <- pastvalue(t - tp)
  }
  
  if (t - tc < t0) {
    # The value of the lagged variables. Setting them equal to 0 means no lag.
    lagc <- y * 0
  } else {
    # if the logical test passses we can calculate the lags
    lagc <- pastvalue(t - tc)
  }
  
  # The system of equations (1)
  P1 <- egg * A * V(S(t))  - egg * lagp[1] * (GammaP(Temp(t)) / GammaP(Temp(t - tp))) * V(S(t - tp)) * PhiP  - mup(S(t)) * P
  I1 <- egg * lagp[1] * (GammaP(Temp(t)) / GammaP(Temp(t - tp))) * V(S(t - tp)) * PhiP - iota * f * I - mui(S(t)) * I
  C1 <- iota * f * I - iota * f * lagc[4] * (GammaC(Temp(t)) / GammaC(Temp(t - tc))) * PhiC - muc(S(t)) * C
  A1 <- ((iota * f * lagc[4] * (GammaC(Temp(t)) / GammaC(Temp(t - tc))) * PhiC) / 2) - mua(S(t)) * A
  
  tp1 <- 1 - GammaP(Temp(t)) / GammaP(Temp(t - tp))
  tc1 <- 1 - GammaC(Temp(t))/GammaC(Temp(t - tc))
  
  PhiP1 <- PhiP * (GammaP(Temp(t)) * mup(S(t - tp))/GammaP(Temp(t - tp)) - mup(S(t)))
  PhiC1 <- PhiC * (GammaC(Temp(t)) * muc(S(t - tc)) / GammaC(Temp(t - tc)) - muc(S(t)))
  
  return(c(A1, P1, tp1, I1, C1, tc1, PhiP1, PhiC1))
}

# create matrix to hold output values
sealiceA <- matrix(NA, nsim, 181)
sealiceP <- matrix(NA, nsim, 181)
sealiceI <- matrix(NA, nsim, 181)
sealiceC <- matrix(NA, nsim, 181)
sealiceTP <- matrix(NA, nsim, 181)
sealiceTC <- matrix(NA, nsim, 181)
sealicePhiP <- matrix(NA, nsim, 181)
sealicePhiC <- matrix(NA, nsim, 181)

# set up progress trackers
pb <- txtProgressBar(min = 0, max = nsim, style = 3)
strt <- Sys.time()

### each loop uses the ith values from the latin hypercube as input values
for(i in 1:nsim){
  
  # Start and Stop Times
  t0 <- 0
  tfin <- 180
  
  # Reproduction Parameters
  eta <- 296 * 2
  epsilon <- 0.0476
  egg <- eta * epsilon
  
  # Temperature Parameters
  a <- SimTemp[i]
  c <- SimSal[i]
  b <- 4
  d <- 2
  
  # Growth rate parameters
  Tref <- 10
  alphap <- 13.01
  alphac <- 11.94
  Betap <- 0.525
  Betac <- 0.177
  
  # Infection Rate Parameters
  f <- 1000
  iota <- 0.01
  
  # Mortality/Viability Parameters
  sigma0e <- -0.458
  sigma1e <- 0.037
  
  sigma0a <- 4.12
  sigma1a <- 0.124
  sigma0p <- 0.4492
  sigma1p <- -0.01484
  sigma0i <- 0.4492
  sigma1i <- -0.01484
  sigma0c <- 4.12
  sigma1c <- 0.124
  
  # List of Parameters
  parameters <- c(egg, iota, a, Betap, Betac, f, V, mup, mui, muc, mua, b, Tref,
                  alphap, alphac, t0, tfin, mup)
  
  # Inital condition.
  A0 <- f
  P0 <- 0
  I0 <- 0
  C0 <- 0
  
  # The Initial Delays (Are "u", where u is such that the integral of gamma from
  # t0-u to t0 is equal to 1)
  gamP <- function(t){ # These gamma functions include Temp for the integrate function
    Temp <- a - b * cos(2 * pi * t / 365)
    gP <- ((Betap * (Temp - Tref + alphap)) / (alphap)) ^ 2
    return(gP)
  }
  
  gamC <- function(t){ # These gamma functions include Temp for the integrate function
    Temp <- a - b * cos(2 * pi * t / 365)
    gC <- ((Betac * (Temp - Tref + alphac)) / (alphac)) ^ 2
    return(gC)
  }
  
  tpINIT <- function(t0){
    IntegralValues <- matrix(NA, 200, 2) # Increase row length if expected delay > 200 days
    for(u in 0:20){
      IntegralValues[u,2] <- integrate(gamP, t0 - u, t0)$value
      IntegralValues[u,1] <- u - 1
    }
    tauP0 <- max(IntegralValues[which(IntegralValues[,2] <= 1)])
    return(tauP0)
  }
  
  tcINIT <- function(t0){
    IntegralValues <- matrix(NA, 200, 2) # Increase row length if expected delay > 200 days
    for(u in 0:200){
      IntegralValues[u,2] <- integrate(gamC, t0 - u, t0)$value
      IntegralValues[u,1] <- u - 1
    }
    tauC0 <- max(IntegralValues[which(IntegralValues[,2] <= 1)])
    return(tauC0)
  }
  
  tp0 <- tpINIT(t0 = t0)
  tc0 <- tcINIT(t0 = t0)
  
  # The Initial Stage Survival Probabilities
  mortalityP <- function(t){
    Salinity <- S(t)
    Mortality <- mup(Salinity)
    return(Mortality)
  }
  
  mortalityC <- function(t){
    Salinity <- S(t)
    Mortality <- muc(Salinity)
    return(Mortality)
  }
  
  PhiP0 <- exp(-(integrate(mortalityP, t0 - tp0, t0)$value))
  PhiC0 <- exp(-(integrate(mortalityC, t0 - tc0, t0)$value))
  
  # List of All Initial Values
  yinit <- c(A0, P0, tp0, I0, C0, tc0, PhiP0, PhiC0)
  
  ## Run Model
  x <- dde(y = yinit, func = lice, parms = parameters, times = seq(t0, tfin, 1), 
           hbsize = 1000000)
  
  # Response is average number of adult females per fish
  sealiceA[i, ] <- x[, 2] / f
  sealiceP[i, ] <- x[, 3] / f
  sealiceI[i, ] <- x[, 5] / f
  sealiceC[i, ] <- x[, 6] / f
  sealiceTP[i, ] <- x[, 4]
  sealiceTC[i, ] <- x[, 7]
  sealicePhiP[i, ] <- x[, 8]
  sealicePhiC[i, ] <- x[, 9]
  
  # Monitoring progress of for loop
  setTxtProgressBar(pb, i)
}

# Closes progress trackers, stops cluster, and reports how long the loop took to run
print(Sys.time() - strt)
close(pb)

# Plot
pdf(file = "ModelDynamics2.pdf")
days <- seq(t0, tfin, 1)
plot(days, sealiceA[1, ], type = "l", col = "blue", lwd = 3, 
     ylab = "Adult Female Sea Lice Per Fish", xlab = "Time (Days)", 
     cex.axis = 1.5, cex.lab = 1.5, ylim = c(0, 100))
lines(days, sealiceA[2, ], col = "black", lwd = 3)
lines(days, sealiceA[3, ], col = "red", lwd = 3)
lines(days, sealiceA[4, ], col = "green", lwd = 3)
legend("topleft", c(expression(paste("6",degree,"C & 15psu")), 
                    expression(paste("6",degree,"C & 25psu")),
                    expression(paste("10",degree,"C & 15psu")),
                    expression(paste("10",degree,"C & 25psu"))), 
       cex = 1.3, lty = c(1, 1, 1, 1), col = c("blue", "black", "red", "green"), 
       lwd = c(3, 3, 3, 3)) 
dev.off()

#######################################
# Persistance/Extinction Figure
#######################################

### Mean annual temperatures and salinities

SimTemp <- c(7, 8, 9, 6, 7, 8, 9, 6, 7, 8, 9, 6, 7, 8, 9, 6)
SimSal=c(14, 16, 18, 20, 20, 14, 16, 18, 18, 20, 14, 16, 16, 18, 20, 14)
ENV <- cbind(SimTemp, SimSal)
ENV
nsim <- length(SimTemp)

### The Model
## Temperature/development
# Temperature seasonality
Temp <- function(t){
  a - b * cos(2 * pi * t / 365)
}

# planktonic delay
GammaP <- function(Temp){
  ((Betap * (Temp - Tref + alphap)) / (alphap)) ^ 2
}

# chalimus delay
GammaC <- function(Temp){
  ((Betac * (Temp - Tref + alphac)) / (alphac)) ^ 2
}

## Salinity/Mortality
# Salinity seasonality
S <- function(t){
  c - d * cos(2 * pi * t / 365)
}

# egg viability fit to data from Johnson & Albright 1991
V <- function(S){
  Ve <- sigma0e + sigma1e * S
  ifelse(Ve < 0, 0, Ve)  
}

# adult mortality based on Connors 2008
mua <- function(S){
  logsurada <- (sigma0a + sigma1a * S)
  suradaest <- exp(logsurada)
  mewa <- 24 / suradaest
  ifelse(mewa > 1, 1, mewa)  
}

# chalimus mortality is similar to adult so we use same eqs
muc <- function(S){
  logsurcha <- (sigma0c + sigma1c * S)
  surchaest <- exp(logsurcha)
  mewc <- 24 / surchaest
  ifelse(mewc > 1, 1, mewc) 
}

# Nauplius mortality from Brooks & Stucchi 2005
mup <- function(S){
  mup <- sigma0p + sigma1p * S
  ifelse(mup < 0, 0, mup)
}

# Copepodid Mortality assumed to be similar to nauplius mortality
mui <- function(S){
  mui <- sigma0i + sigma1i * S
  ifelse(mui < 0, 0, mui)
}

### The DDE system
## Function to calculate the gradient
lice <- function(t, y){
  A <- y[1]
  P <- y[2]
  tp <- y[3]
  I <- y[4]
  C <- y[5]
  tc <- y[6]
  PhiP <- y[7]
  PhiC <- y[8]
  
  # Apply timelags only once enough history has been established:
  
  if (t - tp < t0) {
    # The value of the lagged variables. Setting them equal to 0 means no lag.
    lagp <- y * 0
  } else {
    # if the logical test passses we can calculate the lags
    lagp <- pastvalue(t - tp)
  }
  
  if (t - tc < t0) {
    # The value of the lagged variables. Setting them equal to 0 means no lag.
    lagc <- y * 0
  } else {
    # if the logical test passses we can calculate the lags
    lagc <- pastvalue(t - tc)
  }
  
  # The system of equations (1)
  P1 <- egg * A * V(S(t))  - egg * lagp[1] * (GammaP(Temp(t)) / GammaP(Temp(t - tp))) * V(S(t - tp)) * PhiP  - mup(S(t)) * P
  I1 <- egg * lagp[1] * (GammaP(Temp(t)) / GammaP(Temp(t - tp))) * V(S(t - tp)) * PhiP - iota * f * I - mui(S(t)) * I
  C1 <- iota * f * I - iota * f * lagc[4] * (GammaC(Temp(t)) / GammaC(Temp(t - tc))) * PhiC - muc(S(t)) * C
  A1 <- ((iota * f * lagc[4] * (GammaC(Temp(t)) / GammaC(Temp(t - tc))) * PhiC) / 2) - mua(S(t)) * A
  
  tp1 <- 1 - GammaP(Temp(t)) / GammaP(Temp(t - tp))
  tc1 <- 1 - GammaC(Temp(t))/GammaC(Temp(t - tc))
  
  PhiP1 <- PhiP * (GammaP(Temp(t)) * mup(S(t - tp))/GammaP(Temp(t - tp)) - mup(S(t)))
  PhiC1 <- PhiC * (GammaC(Temp(t)) * muc(S(t - tc)) / GammaC(Temp(t - tc)) - muc(S(t)))
  
  return(c(A1, P1, tp1, I1, C1, tc1, PhiP1, PhiC1))
}

# create matrix to hold output values
sealiceA <- matrix(NA, nsim, 1461)
sealiceP <- matrix(NA, nsim, 1461)
sealiceI <- matrix(NA, nsim, 1461)
sealiceC <- matrix(NA, nsim, 1461)
sealiceTP <- matrix(NA, nsim, 1461)
sealiceTC <- matrix(NA, nsim, 1461)
sealicePhiP <- matrix(NA, nsim, 1461)
sealicePhiC <- matrix(NA, nsim, 1461)

# set up progress trackers
pb <- txtProgressBar(min = 0, max = nsim, style = 3)
strt <- Sys.time()

### each loop uses the ith values from the latin hypercube as input values
for(i in 1:nsim){
  
  # Start and Stop Times
  t0 <- 0
  tfin <- 1460
  
  # Reproduction Parameters
  eta <- 296 * 2
  epsilon <- 0.0476
  egg <- eta * epsilon
  
  # Temperature Parameters
  a <- ENV[i, 1]
  c <- ENV[i, 2]
  b <- 4
  d <- 2
  
  # Growth rate parameters
  Tref <- 10
  alphap <- 13.01
  alphac <- 11.94
  Betap <- 0.525
  Betac <- 0.177
  
  # Infection Rate Parameters
  f <- 1000
  iota <- 0.01
  
  # Mortality/Viability Parameters
  sigma0e <- -0.458
  sigma1e <- 0.037
  
  sigma0a <- 4.12
  sigma1a <- 0.124
  sigma0p <- 0.4492
  sigma1p <- -0.01484
  sigma0i <- 0.4492
  sigma1i <- -0.01484
  sigma0c <- 4.12
  sigma1c <- 0.124
  
  # Paremeter List
  parameters <- c(egg, iota, a, Betap, Betac, f, V, mup, mui, muc, mua, b, Tref,
                  alphap, alphac, t0, tfin, mup)
  
  ## Inital condition.
  A0 <- f
  P0 <- 0
  I0 <- 0
  C0 <- 0
  
  # The Initial Delays (Are "u", where u is such that the integral of gamma from
  # t0-u to t0 is equal to 1)
  gamP <- function(t){ # These gamma functions include Temp for the integrate function
    Temp <- a - b * cos(2 * pi * t / 365)
    gP <- ((Betap * (Temp - Tref + alphap)) / (alphap)) ^ 2
    return(gP)
  }
  
  gamC <- function(t){ # These gamma functions include Temp for the integrate function
    Temp <- a - b * cos(2 * pi * t / 365)
    gC <- ((Betac * (Temp - Tref + alphac)) / (alphac)) ^ 2
    return(gC)
  }
  
  tpINIT <- function(t0){
    IntegralValues <- matrix(NA, 200, 2) # Increase row length if expected delay > 200 days
    for(u in 0:20){
      IntegralValues[u,2] <- integrate(gamP, t0 - u, t0)$value
      IntegralValues[u,1] <- u - 1
    }
    tauP0 <- max(IntegralValues[which(IntegralValues[,2] <= 1)])
    return(tauP0)
  }
  
  tcINIT <- function(t0){
    IntegralValues <- matrix(NA, 200, 2) # Increase row length if expected delay > 200 days
    for(u in 0:200){
      IntegralValues[u,2] <- integrate(gamC, t0 - u, t0)$value
      IntegralValues[u,1] <- u - 1
    }
    tauC0 <- max(IntegralValues[which(IntegralValues[,2] <= 1)])
    return(tauC0)
  }
  
  tp0 <- tpINIT(t0 = t0)
  tc0 <- tcINIT(t0 = t0)
  
  # The Initial Stage Survival Probabilities
  mortalityP <- function(t){
    Salinity <- S(t)
    Mortality <- mup(Salinity)
    return(Mortality)
  }
  
  mortalityC <- function(t){
    Salinity <- S(t)
    Mortality <- muc(Salinity)
    return(Mortality)
  }
  
  PhiP0 <- exp(-(integrate(mortalityP, t0 - tp0, t0)$value))
  PhiC0 <- exp(-(integrate(mortalityC, t0 - tc0, t0)$value))
  
  # List of All Initial Values
  yinit <- c(A0, P0, tp0, I0, C0, tc0, PhiP0, PhiC0)
  
  ## Run Model
  x <- dde(y = yinit, func = lice, parms = parameters, times = seq(t0, tfin, 1),
           hbsize = 1000000)
  
  #Response is average number of adult females per fish
  sealiceA[i, ] <- x[, 2] / f
  sealiceP[i, ] <- x[, 3] / f
  sealiceI[i, ] <- x[, 5] / f
  sealiceC[i, ] <- x[, 6] / f
  sealiceTP[i, ] <- x[, 4]
  sealiceTC[i, ] <- x[, 7]
  sealicePhiP[i, ] <- x[, 8]
  sealicePhiC[i, ] <- x[, 9]
  
  #Monitoring progress of for loop
  setTxtProgressBar(pb, i)
}

#Closes progress trackers, stops cluster, and reports how long the loop took to run
print(Sys.time() - strt)
close(pb)

## Making Persistence/Extinction Figure
# Deterimining if population grew towards infinity or shrank to 0
longtermresults <- cbind(ENV[, 1], ENV[, 2], sealiceA[, 1461])
empty.col <- matrix(NA, 16, 1)
gglongterm.data <- cbind(longtermresults, empty.col)
for(i in 1:length(gglongterm.data[, 3])){
  if(gglongterm.data[i, 3] >= 1){
    gglongterm.data[i, 4] <- 1
    } else if(gglongterm.data[i, 3] < 1){
    gglongterm.data[i, 4] <- 0
  }
}

gglongterm.data <- as.data.frame(gglongterm.data)
names(gglongterm.data) <- c("temperature", "salinity", "population", "dynamic")

# Plot
gglongterm <- qplot(temperature, salinity, data = gglongterm.data, colour = as.factor(dynamic), 
      shape = as.factor(dynamic), size = I(10)) +
      scale_color_discrete(name = "Dynamics", labels = c("Decay", "Growth")) + 
      scale_shape_discrete(name = "Dynamics", labels = c("Decay", "Growth")) +
      labs(x = expression(Temperature ~ degree ~ C), y = expression(Salinity (psu))) 

ggsave(filename = "gglongterm.pdf", plot = gglongterm, width = 9.39, height = 9.67)

