###########################
# R0(t) for farm 3 in Marty Data
###########################

# Clearing Workspace and loading packages
# rm(list = ls())
# setwd() # Set this to preferred working directory 
library(PBSddesolve)

# Set Start Dates
start.date <- seq(1, 365, 30)
nsim <- length(start.date)

### The Model
## Temperature/Development
# Temperature Seasonality
Temp <- function(t){
  9.07 - 0.62 * sin(2 * pi * t / 365) - 2.52 * cos( 2 * pi * t / 365)
}

# planktonic delay
GammaP <- function(Temp){
  ((Betap * (Temp - Tref + alphap)) / (alphap)) ^ 2
}

# chalimus delay
GammaC <- function(Temp){
  ((Betac * (Temp - Tref + alphac)) / (alphac)) ^ 2
}

## Salinity/Mortality
# Salinity Seasonality
S <- function(t){
  24.76 + 1.88 * sin(2 * pi * t / 365) + 7.08 * cos(2 * pi * t / 365)
}

# egg viability fit to data from Johnson & Albright 1991
V <- function(S){
  Ve <- sigma0e+sigma1e*S
  ifelse(Ve < 0, 0, Ve)  
}

# adult mortality based on Connors 2008
mua <- function(S){
  logsurada <- (sigma0a + sigma1a * S)
  suradaest <- exp(logsurada)
  mewa <- 24 / suradaest
  ifelse(mewa > 1, 1, mewa)  
}

# chalimus mortality is similar to adult so we use same eqs
muc <- function(S){
  logsurcha <- (sigma0c + sigma1c * S)
  surchaest <- exp(logsurcha)
  mewc <- 24 / surchaest
  ifelse(mewc > 1, 1, mewc) 
}

# Nauplius mortality from Brooks & Stucchi 2005
mup <- function(S){
  mup <- sigma0p + sigma1p * S
  ifelse(mup < 0, 0, mup)
}

# Copepodid Mortality assumed to be similar to nauplius mortality
mui <- function(S){
  mui <- sigma0i + sigma1i * S
  ifelse(mui < 0, 0, mui)
}

### The DDE system
## Function to calculate the gradient
lice <- function(t, y){
  A <- y[1]
  P <- y[2]
  tp <- y[3]
  I <- y[4]
  C <- y[5]
  tc <- y[6]
  PhiP <- y[7]
  PhiC <- y[8]
  R <- y[9]
  
  # Apply timelags only once enough history has been established:
  
  if (t-tp < t0) {
    # The value of the lagged variables. Setting them equal to 0 means no lag.
    lagp <- y * 0 
  } else {
    # if the logical test passses we can calculate the lags
    lagp <- pastvalue(t - tp)
  }
  
  if (t - tc < t0) {
    # The value of the lagged variables. Setting them equal to 0 means no lag.
    lagc <- y * 0
  } else {
    # if the logical test passses we can calculate the lags
    lagc <- pastvalue(t - tc)
  }
    
  # The system of equations (1)
  P1 <- egg * A * V(S(t)) - egg * lagp[1] * V(S(t - tp)) * (GammaP(Temp(t)) / GammaP(Temp(t - tp))) * PhiP - mup(S(t)) * P
  I1 <- egg * lagp[1]*V(S(t - tp)) * (GammaP(Temp(t)) / GammaP(Temp(t - tp))) * PhiP - iota * f * I - mui(S(t)) * I
  C1 <- iota * f * I - iota * f * lagc[4] * (GammaC(Temp(t)) / GammaC(Temp(t - tc))) * PhiC - muc(S(t)) * C
  A1 <- - mua(S(t)) * A
  
  R1 <- ((iota * f * lagc[4] * (GammaC(Temp(t)) / GammaC(Temp(t - tc))) * PhiC) / 2)

  tp1 <- 1 - GammaP(Temp(t)) / GammaP(Temp(t - tp))
  tc1 <- 1 - GammaC(Temp(t)) / GammaC(Temp(t - tc))
  
  PhiP1 <- PhiP * (GammaP(Temp(t)) * mup(S(t - tp)) / GammaP(Temp(t - tp)) - mup(S(t)))
  PhiC1 <- PhiC * (GammaC(Temp(t)) * muc(S(t - tc)) / GammaC(Temp(t - tc)) - muc(S(t)))
  
  return(c(A1, P1, tp1, I1, C1, tc1, PhiP1, PhiC1, R1))
}

# create matrix to hold output values

r.nought <- matrix(NA, nsim, 1)

# set up progress trackers
pb <- txtProgressBar(min = 0, max = nsim, style = 3)
strt <- Sys.time()

### Each loop starts at a new date
for(i in 1:nsim){
  
  # Start and Stop Times
  t0 <- start.date[i]
  tfin <- start.date[i] + 1460
  
  # Reproduction Parameters
  eta <- 296 * 2
  epsilon <- 0.0476
  egg <- eta * epsilon
  
  # Growth rate parameters
  Tref <- 10
  alphap <- 13.01
  alphac <- 11.94
  Betap <- 0.525
  Betac <- 0.177
  
  # Infection Rate Parameters
  f <- 1000
  iota <- 0.01
  
  # Mortality/Viability Parameters
  sigma0e <- -0.458
  sigma1e <- 0.037
  
  sigma0a <- 4.12
  sigma1a <- 0.124
  sigma0p <- 0.4492
  sigma1p <- -0.01484
  sigma0i <- 0.4492
  sigma1i <- -0.01484
  sigma0c <- 4.12
  sigma1c <- 0.124
  
  # List of Parameters
  parameters <- c(egg, iota, Betap, Betac, f, V, mup, mui, muc, mua, Tref,
                  alphap, alphac, t0, tfin, mup)
  
  # Inital condition.
  A0 <- 1
  P0 <- 0
  I0 <- 0
  C0 <- 0
  R0 <- 0

  # The Initial Delays (Are "u", where u is such that the integral of gamma from
  # t0-u to t0 is equal to 1)
  gamP <- function(t){ # These gamma functions include Temp for the integrate function
    Temp <- 9.07 - 0.62 * sin(2 * pi * t / 365) - 2.52 * cos( 2 * pi * t / 365)
    gP <- ((Betap * (Temp - Tref + alphap)) / (alphap)) ^ 2
    return(gP)
  }
  
  gamC <- function(t){ # These gamma functions include Temp for the integrate function
    Temp <- 9.07 - 0.62 * sin(2 * pi * t / 365) - 2.52 * cos( 2 * pi * t / 365)
    gC <- ((Betac * (Temp - Tref + alphac)) / (alphac)) ^ 2
    return(gC)
  }
  
  tpINIT <- function(t0){
    IntegralValues <- matrix(NA, 200, 2) # Increase row length if expected delay > 200 days
    for(u in 0:20){
      IntegralValues[u,2] <- integrate(gamP, t0 - u, t0)$value
      IntegralValues[u,1] <- u - 1
    }
    tauP0 <- max(IntegralValues[which(IntegralValues[,2] <= 1)])
    return(tauP0)
  }
  
  tcINIT <- function(t0){
    IntegralValues <- matrix(NA, 200, 2) # Increase row length if expected delay > 200 days
    for(u in 0:200){
      IntegralValues[u,2] <- integrate(gamC, t0 - u, t0)$value
      IntegralValues[u,1] <- u - 1
    }
    tauC0 <- max(IntegralValues[which(IntegralValues[,2] <= 1)])
    return(tauC0)
  }
  
  tp0 <- tpINIT(t0 = t0)
  tc0 <- tcINIT(t0 = t0)
  
  # The Initial Stage Survival Probabilities
  mortalityP <- function(t){
    Salinity <- S(t)
    Mortality <- mup(Salinity)
    return(Mortality)
  }
  
  mortalityC <- function(t){
    Salinity <- S(t)
    Mortality <- muc(Salinity)
    return(Mortality)
  }
  
  PhiP0 <- exp(-(integrate(mortalityP, t0 - tp0, t0)$value))
  PhiC0 <- exp(-(integrate(mortalityC, t0 - tc0, t0)$value))
  
  # List of All Initial Values
  yinit <- c(A0, P0, tp0, I0, C0, tc0, PhiP0, PhiC0, R0)
  
  ## Run Model
  x <- dde(y = yinit, func = lice, parms = parameters, times = seq(t0, tfin, 1),
           hbsize = 1000000)
  
  r.nought[i] <- max(x[, 10]) # R0(t)
  
  plot(x[, 1], x[, 10]) # Shows R(t) converge to its final value
   
  ## tau_hatP and tau_hatC
  if (i == 1){
    tau.hatP <- matrix(NA, length(x[, 1]), 1)
    tau.hatC <- matrix(NA, length(x[, 1]), 1)
    for(j in 365:length(x[, 1])){
      tau.hatP[j] <- x[j, 1] - (x[j, 1] - x[j, 4])
      tau.hatC[j] <- x[j, 1] - (x[j, 1] - x[j, 7])    
    }
    daysP <- x[, 1] - tau.hatP - 365
    daysC <- x[, 1] - tau.hatC - 365
    tau.hats <- as.data.frame(cbind(x[, 1], tau.hatP, tau.hatC, daysP, daysC))
    names(tau.hats) <- c("DoY", "tauhatP.BC", "tauhatC.BC", "daysP.BC", "daysC.BC")
    write.csv(tau.hats, "/Users/hurford1/Desktop/outputs/tauhatsBC.csv", 
              row.names = FALSE)
  }
  
  ## Mortalities
  if (i == 1){
    mortality.p <- mup(S(x[, 1]))
    mortality.a <- mua(S(x[, 1]))
    time.and.mortality.BC <- as.data.frame(cbind(x[, 1], mortality.p, mortality.a))
    names(time.and.mortality.BC) <- c("DoY", "mortality.p.BC", "mortality.a.BC")
    write.csv(time.and.mortality.BC, "mortalityBC.csv", row.names = FALSE)
  }
  
  # Monitoring progress of for loop
  setTxtProgressBar(pb, i)
}

# Closes progress trackers, stops cluster, and reports how long the loop took to run
print(Sys.time() - strt)
close(pb)

## Saving R0(t) values
write.csv(r.nought, file = "R0_BC", row.names = FALSE)

###########################
# R0(t) for NL
###########################

# Set Start Dates
start.date <- seq(1, 365, 30)
nsim <- length(start.date)

### The Model
## Temperature/Development
# Temperature Seasonality
Temp <- function(t){
  Temperature <- 6.47 - 4.09 * sin (2 * pi * t / 365) - 1.84 * cos(2 * pi * t / 365)
  return(Temperature)
}

# planktonic delay
GammaP <- function(Temp){
  gP <- ((Betap * (Temp - Tref + alphap)) / (alphap)) ^ 2
  return(gP) 
}

# chalimus delay
GammaC <- function(Temp){
  ((Betac * (Temp - Tref + alphac)) / (alphac)) ^ 2
}

## Salinity/Mortality
# Parameters for Salinity seasonality
S <- function(t){
  22.89 - 1.39 * sin(2 * pi * t / 365) - 0.19 * cos(2 * pi * t / 365)
}

# egg viability fit to data from Johnson & Albright 1991
V <- function(S){
  Ve <- sigma0e + sigma1e * S
  ifelse(Ve < 0, 0, Ve)  
}

# adult mortality based on Connors 2008
mua <- function(S){
  logsurada <- (sigma0a + sigma1a * S)
  suradaest <- exp(logsurada)
  mewa <- 24 / suradaest
  ifelse(mewa > 1, 1, mewa)  
}

# chalimus mortality is similar to adult so we use same eqs
muc <- function(S){
  logsurcha <- (sigma0c + sigma1c * S)
  surchaest <- exp(logsurcha)
  mewc <- 24 / surchaest
  ifelse(mewc > 1, 1, mewc) 
}

# Nauplius mortality from Brooks & Stucchi 2005
mup <- function(S){
  mup <- sigma0p + sigma1p * S
  ifelse(mup < 0, 0, mup)
}

# Copepodid Mortality assumed to be similar to nauplius mortality
mui <- function(S){
  mui <- sigma0i + sigma1i * S
  ifelse(mui < 0, 0, mui)
}

### The DDE system
## Function to calculate the gradient
lice <- function(t, y){
  A <- y[1]
  P <- y[2]
  tp <- y[3]
  I <- y[4]
  C <- y[5]
  tc <- y[6]
  PhiP <- y[7]
  PhiC <- y[8]
  R <- y[9]
  
  # Apply timelags only once enough history has been established:
  
  if (t - tp < t0) {
    # The value of the lagged variables. Setting them equal to 0 means no lag.
    lagp <- y * 0
  } else {
    # if the logical test passses we can calculate the lags
    lagp <- pastvalue(t - tp)
  }
  
  if (t - tc < t0) {
    # The value of the lagged variables. Setting them equal to 0 means no lag.
    lagc <- y * 0
  } else {
    # if the logical test passses we can calculate the lags
    lagc <- pastvalue(t - tc)
  }
  
  # The system of equations (1)
  P1 <- egg * A * V(S(t)) - egg * lagp[1] * V(S(t - tp)) * (GammaP(Temp(t)) / GammaP(Temp(t - tp))) * PhiP - mup(S(t)) * P
  I1 <- egg * lagp[1] * V(S(t - tp)) * (GammaP(Temp(t)) / GammaP(Temp(t - tp))) * PhiP - iota * f * I - mui(S(t)) * I
  C1 <- iota * f * I - iota * f * lagc[4] * (GammaC(Temp(t)) / GammaC(Temp(t - tc))) * PhiC - muc(S(t)) * C
  A1 <- - mua(S(t)) * A
  
  R1 <- ((iota * f * lagc[4] * (GammaC(Temp(t)) / GammaC(Temp(t - tc))) * PhiC) / 2)
  
  tp1 <- 1 - GammaP(Temp(t)) / GammaP(Temp(t - tp))
  tc1 <- 1 - GammaC(Temp(t)) / GammaC(Temp(t - tc))
  
  PhiP1 <- PhiP * (GammaP(Temp(t)) * mup(S(t - tp)) / GammaP(Temp(t - tp)) - mup(S(t)))
  PhiC1 <- PhiC * (GammaC(Temp(t)) * muc(S(t-tc)) / GammaC(Temp(t - tc)) - muc(S(t)))
  
  return(c(A1, P1, tp1, I1, C1, tc1, PhiP1, PhiC1, R1))
}

#create matrix to hold output values

r.nought <- matrix(NA, nsim, 1)

# set up progress trackers
pb <- txtProgressBar(min = 0, max = nsim, style = 3)
strt <- Sys.time()

# each loop starts the model 30 days later
for(i in 1:nsim){
  
  # Start and Stop Times
  t0 <- start.date[i]
  tfin <- start.date[i] + 1460
  
  # Reproduction Parameters
  eta <- 296 * 2
  epsilon <- 0.0476
  egg <- eta * epsilon
  
  
  # Growth rate parameters
  Tref <- 10
  alphap <- 13.01
  alphac <- 11.94
  Betap <- 0.525
  Betac <- 0.177
  
  # Infection Rate Parameters
  f <- 1000
  iota <- 0.01
  
  # Mortality/Viability Parameters
  sigma0e <- -0.458
  sigma1e <- 0.037
  
  sigma0a <- 4.12
  sigma1a <- 0.124
  sigma0p <- 0.4492
  sigma1p <- -0.01484
  sigma0i <- 0.4492
  sigma1i <- -0.01484
  sigma0c <- 4.12
  sigma1c <- 0.124
  
  # List of Parameters
  parameters <- c(egg, iota, Betap, Betac, f, V, mup, mui, muc, mua, Tref, 
                  alphap, alphac, t0, tfin, mup)
  
  ## Inital condition.
  A0 <- 1
  P0 <- 0
  I0 <- 0
  C0 <- 0
  R0 <- 0
  
  # The Initial Delays (Are "u", where u is such that the integral of gamma from
  # t0-u to t0 is equal to 1)
  gamP <- function(t){ # These gamma functions include Temp for the integrate function
    Temp <- 6.47 - 4.09 * sin (2 * pi * t / 365) - 1.84 * cos(2 * pi * t / 365)
    gP <- ((Betap * (Temp - Tref + alphap)) / (alphap)) ^ 2
    return(gP)
  }
  
  gamC <- function(t){ # These gamma functions include Temp for the integrate function
    Temp <- 6.47 - 4.09 * sin (2 * pi * t / 365) - 1.84 * cos(2 * pi * t / 365)
    gC <- ((Betac * (Temp - Tref + alphac)) / (alphac)) ^ 2
    return(gC)
  }
  
  tpINIT <- function(t0){
    IntegralValues <- matrix(NA, 200, 2) # Increase row length if expected delay > 200 days
    for(u in 0:20){
      IntegralValues[u,2] <- integrate(gamP, t0 - u, t0)$value
      IntegralValues[u,1] <- u - 1
    }
    tauP0 <- max(IntegralValues[which(IntegralValues[,2] <= 1)])
    return(tauP0)
  }
  
  tcINIT <- function(t0){
    IntegralValues <- matrix(NA, 200, 2) # Increase row length if expected delay > 200 days
    for(u in 0:200){
      IntegralValues[u,2] <- integrate(gamC, t0 - u, t0)$value
      IntegralValues[u,1] <- u - 1
    }
    tauC0 <- max(IntegralValues[which(IntegralValues[,2] <= 1)])
    return(tauC0)
  }
  
  tp0 <- tpINIT(t0 = t0)
  tc0 <- tcINIT(t0 = t0)
  
  # The Initial Stage Survival Probabilities
  mortalityP <- function(t){
    Salinity <- S(t)
    Mortality <- mup(Salinity)
    return(Mortality)
  }
  
  mortalityC <- function(t){
    Salinity <- S(t)
    Mortality <- muc(Salinity)
    return(Mortality)
  }
  
  PhiP0 <- exp(-(integrate(mortalityP, t0 - tp0, t0)$value))
  PhiC0 <- exp(-(integrate(mortalityC, t0 - tc0, t0)$value))
  
  # List of All Initial Values
  yinit <- c(A0, P0, tp0, I0, C0, tc0, PhiP0, PhiC0, R0)
  
  ## Run Model
  x <- dde(y = yinit, func = lice, parms = parameters, times = seq(t0, tfin, 1), 
           hbsize = 1000000)
  
  r.nought[i] <- max(x[, 10]) # R0(t)
  
  plot(x[, 1], x[, 10]) # Shows R(t) converge to final value
  
  ## tau_hatP and tau_hatC
  if (i == 1){
    tau.hatP <- matrix(NA, length(x[, 1]), 1)
    tau.hatC <- matrix(NA, length(x[, 1]), 1)
    for(j in 365:length(x[, 1])){
      tau.hatP[j] <- x[j, 1] - (x[j, 1] - x[j, 4])
      tau.hatC[j] <- x[j, 1] - (x[j, 1] - x[j, 7])    
    }
    daysP <- x[, 1] - tau.hatP - 365
    daysC <- x[, 1] - tau.hatC - 365
    tau.hats <- as.data.frame(cbind(x[, 1], tau.hatP, tau.hatC, daysP, daysC))
    names(tau.hats) <- c("DoY", "tauhatP.NL", "tauhatC.NL", "daysP.NL", "daysC.NL")
    write.csv(tau.hats, "/Users/hurford1/Desktop/outputs/tauhatsNL.csv", 
              row.names = FALSE)
  }
  
  ## mortalities
  if (i == 1){
    mortality.p <- mup(S(x[, 1]))
    mortality.a <- mua(S(x[, 1]))
    time.and.mortality.NL <- as.data.frame(cbind(x[, 1], mortality.p, mortality.a))
    names(time.and.mortality.NL) <- c("DoY", "mortality.p.NL", "mortality.a.NL")
    write.csv(time.and.mortality.NL, "mortalityNL.csv", row.names = FALSE)
  }
  
  # Monitoring progress of for loop
  setTxtProgressBar(pb, i)
}

# Closes progress trackers, stops cluster, and reports how long the loop took to run
print(Sys.time() - strt)
close(pb)

# Save R0(t) values
write.csv(r.nought, file = "R0_nova", row.names = FALSE)

