###########################
# Latin Hypercube Sampling
###########################
# Clearing Workspace and loading packages
# rm(list = ls())
# setwd() # Set this to preferred working directory 
library(lhs)
library(DoE.wrapper)
library(triangle)
library(PBSddesolve)
library(sensitivity)
library(ggplot2)
library(cowplot)

#### Step 1: Create a latin hypercube sample

# nsim should be at least 4/3 k
nsim <- 300
latin <- lhs.design(nruns = nsim, nfactors = 7, factor.names = list(eta = c(0, 1),
                    epsilon = c(0, 1), iota = c(0, 1), a = c(0, 1), c = c(0, 1), 
                    Betap = c(0, 1), Betac = c(0, 1)), seed = 3)
head(latin)

# converts to correct distribution functions
latinframe <- data.frame(latin)
# PDF from Heuch et al 2000
latinframe$eta <- qnorm(latin$eta, 296 * 2, 100 * 2)
# PDF from Boxaspen 2006
latinframe$epsilon <- qunif(latin$epsilon, 0.0476, 0.0576)
# PDF from Groner et al 2014 
latinframe$iota <- qtriangle(latin$iota, 0.001, 0.9, 0.01)
# PDF estimated from Beamish and Jones 
latinframe$a <- qunif(latin$a, 7, 10)
latinframe$c <- qunif(latin$c, 20, 30)
# PDFs from Stien et al 2005
latinframe$Betap <- qnorm(latin$Betap, 0.525, 0.017)
latinframe$Betac <- qnorm(latin$Betac, 0.177, 0.006)

# The full latin hypercube
head(latinframe)
write.csv(latinframe, file = "latinframe", 
          row.names = FALSE)

####Step 2: Run Model with Latin Values of Parameters

## Temperature/Development
# Temperature seasonality
Temp <- function(t){
  a - b * cos(2 * pi * t / 365)
}

# planktonic delay
GammaP <- function(Temp){
  ((Betap * (Temp - Tref + alphap)) / (alphap)) ^ 2
}

# chalimus delay
GammaC <- function(Temp){
  ((Betac * (Temp - Tref + alphac)) / (alphac)) ^ 2
}

### Salinity/Mortality
# Salinity seasonality
S <- function(t){
  c - d * cos(2 * pi * t / 365)
}

# egg viability fit to data from Johnson & Albright 1991
V <- function(S){
  Ve <- sigma0e + sigma1e * S
  ifelse(Ve < 0, 0, Ve)  
}

# adult mortality based on Connors 2008
mua <- function(S){
  logsurada <- (sigma0a + sigma1a * S)
  suradaest <- exp(logsurada)
  mewa <- 24 / suradaest
  ifelse(mewa > 1, 1, mewa)  
}

# chalimus mortality is similar to adult so we use same eqs
muc <- function(S){
  logsurcha <- (sigma0c + sigma1c * S)
  surchaest <- exp(logsurcha)
  mewc <- 24 / surchaest
  ifelse(mewc > 1, 1, mewc) 
}

# Nauplius mortality from Brooks & Stucchi 2005
mup <- function(S){
  mup <- sigma0p + sigma1p * S
  ifelse(mup < 0, 0, mup)
}

# Copepodid Mortality assumed to be similar to nauplius mortality
mui <- function(S){
  mui <- sigma0i + sigma1i * S
  ifelse(mui < 0, 0, mui)
}

### The DDE system
## Function to calculate the gradient
lice <- function(t, y){
  A <- y[1]
  P <- y[2]
  tp <- y[3]
  I <- y[4]
  C <- y[5]
  tc <- y[6]
  PhiP <- y[7]
  PhiC <- y[8]
  
  # Apply timelags only once enough history has been established:
  
  if (t - tp < t0) {
    # The value of the lagged variables. Setting them equal to 0 means no lag.
    lagp <- y * 0
  } else {
    # if the logical test passses we can calculate the lags
    lagp <- pastvalue(t - tp)
  }
  
  if (t - tc < t0) {
    # The value of the lagged variables. Setting them equal to 0 means no lag.
    lagc <- y * 0
  } else {
    # if the logical test passses we can calculate the lags
    lagc <- pastvalue(t - tc)
  }
   
  # The system of equations (1)
  P1 <- egg * A * V(S(t))  - egg * lagp[1] * (GammaP(Temp(t)) / GammaP(Temp(t - tp))) * V(S(t - tp)) * PhiP  - mup(S(t)) * P
  I1 <- egg * lagp[1] * (GammaP(Temp(t)) / GammaP(Temp(t - tp))) * V(S(t - tp)) * PhiP - iota * f * I - mui(S(t)) * I
  C1 <- iota * f * I - iota * f * lagc[4] * (GammaC(Temp(t)) / GammaC(Temp(t - tc))) * PhiC - muc(S(t)) * C
  A1 <- ((iota * f * lagc[4] * (GammaC(Temp(t)) / GammaC(Temp(t - tc))) * PhiC) / 2) - mua(S(t)) * A
  
  tp1 <- 1 - GammaP(Temp(t)) / GammaP(Temp(t - tp))
  tc1 <- 1 - GammaC(Temp(t))/GammaC(Temp(t - tc))
  
  PhiP1 <- PhiP * (GammaP(Temp(t)) * mup(S(t - tp))/GammaP(Temp(t - tp)) - mup(S(t)))
  PhiC1 <- PhiC * (GammaC(Temp(t)) * muc(S(t - tc)) / GammaC(Temp(t - tc)) - muc(S(t)))
  
  return(c(A1, P1, tp1, I1, C1, tc1, PhiP1, PhiC1))
}

## Create matrix to hold output values
# Create matrix names for 18 time points for sensitivity analysis.
seadata.num <- matrix(NA, nsim, 1)
for( i in 1:18 ){ 
  matrix.name <- paste( "seadata",i*10, sep="")
  assign( matrix.name, seadata.num, envir = .GlobalEnv)
}

# Matrix for individual variables
seadatatot <- matrix(NA, nsim, 191)

#set up progress trackers
pb <- txtProgressBar(min = 0, max = nsim, style = 3)
strt <- Sys.time()

### each loop uses the ith values from the latin hypercube as input values
for(i in 1:nsim){
  ## Latin parameters
  eta <- latinframe[i, 1]
  epsilon <- latinframe[i, 2]
  iota <- latinframe[i, 3]
  a <- latinframe[i, 4]
  c <- latinframe[i, 5]
  Betap <- latinframe[i, 6]
  Betac <- latinframe[i, 7]
  
  # eggs
  egg <- eta * epsilon
  
  # static parameters
  b <- 4
  d <- 2
  
  Tref <- 10
  alphap <- 13.01
  alphac <- 11.94
  
  t0 <- 0
  tfin <- 190
  
  f <- 1000
  
  sigma0e <- -0.458
  sigma1e <- 0.037
  
  sigma0a <- 4.12
  sigma1a <- 0.124
  sigma0p <- 0.4492
  sigma1p <- -0.01484
  sigma0i <- 0.4492
  sigma1i <- -0.01484
  sigma0c <- 4.12
  sigma1c <- 0.124
  
  # Parameter List
  parameters <- c(egg, iota, a, Betap, Betac, f, V, mup, mui, muc, mua, b, Tref,
                  alphap, alphac, t0, tfin, mup)
  
  ## Inital condition.
  A0 <- f
  P0 <- 0
  I0 <- 0
  C0 <- 0
  
  # The Initial Delays (Are "u", where u is such that the integral of gamma from
  # t0-u to t0 is equal to 1)
  gamP <- function(t){ # These gamma functions include Temp for the integrate function
    Temp <- a - b * cos(2 * pi * t / 365)
    gP <- ((Betap * (Temp - Tref + alphap)) / (alphap)) ^ 2
    return(gP)
  }
  
  gamC <- function(t){ # These gamma functions include Temp for the integrate function
    Temp <- a - b * cos(2 * pi * t / 365)
    gC <- ((Betac * (Temp - Tref + alphac)) / (alphac)) ^ 2
    return(gC)
  }
  
  tpINIT <- function(t0){
    IntegralValues <- matrix(NA, 200, 2) # Increase row length if expected delay > 200 days
    for(u in 0:20){
      IntegralValues[u,2] <- integrate(gamP, t0 - u, t0)$value
      IntegralValues[u,1] <- u - 1
    }
    tauP0 <- max(IntegralValues[which(IntegralValues[,2] <= 1)])
    return(tauP0)
  }
  
  tcINIT <- function(t0){
    IntegralValues <- matrix(NA, 200, 2) # Increase row length if expected delay > 200 days
    for(u in 0:200){
      IntegralValues[u,2] <- integrate(gamC, t0 - u, t0)$value
      IntegralValues[u,1] <- u - 1
    }
    tauC0 <- max(IntegralValues[which(IntegralValues[,2] <= 1)])
    return(tauC0)
  }
  
  tp0 <- tpINIT(t0 = t0)
  tc0 <- tcINIT(t0 = t0)
  
  # The Initial Stage Survival Probabilities
  mortalityP <- function(t){
    Salinity <- S(t)
    Mortality <- mup(Salinity)
    return(Mortality)
  }
  
  mortalityC <- function(t){
    Salinity <- S(t)
    Mortality <- muc(Salinity)
    return(Mortality)
  }
  
  PhiP0 <- exp(-(integrate(mortalityP, t0 - tp0, t0)$value))
  PhiC0 <- exp(-(integrate(mortalityC, t0 - tc0, t0)$value))
  
  # List of All Initial Values
  yinit <- c(A0, P0, tp0, I0, C0, tc0, PhiP0, PhiC0)
  
  ## Run Model
  x <- dde(y = yinit, func = lice, parms = parameters, times = seq(t0, tfin, 1),
           hbsize = 1000000)
  
  #Response is average number of adult females per fish
  seadata10[[i]] <- x[11, 2] / f
  seadata20[[i]] <- x[21, 2] / f
  seadata30[[i]] <- x[31, 2] / f
  seadata40[[i]] <- x[41, 2] / f
  seadata50[[i]] <- x[51, 2] / f
  seadata60[[i]] <- x[61, 2] / f
  seadata70[[i]] <- x[71, 2] / f
  seadata80[[i]] <- x[81, 2] / f
  seadata90[[i]] <- x[91, 2] / f
  seadata100[[i]] <- x[101, 2] / f
  seadata110[[i]] <- x[111, 2] / f
  seadata120[[i]] <- x[121, 2] / f
  seadata130[[i]] <- x[131, 2] / f
  seadata140[[i]] <- x[141, 2] / f
  seadata150[[i]] <- x[151, 2] / f
  seadata160[[i]] <- x[161, 2] / f
  seadata170[[i]] <- x[171, 2] / f
  seadata180[[i]] <- x[181, 2] / f
  seadatatot[i, ] <- x[, 2] / f
  
  #Monitoring progress of for loop
  setTxtProgressBar(pb, i)
}

#Closes progress trackers, stops cluster, and reports how long the loop took to run
print(Sys.time() - strt)
close(pb)

## Saving results to files
write.csv(seadata10, file = "seadata10", row.names = FALSE)
write.csv(seadata20, file = "seadata20", row.names = FALSE)
write.csv(seadata30, file = "seadata30", row.names = FALSE)
write.csv(seadata40, file = "seadata40", row.names = FALSE)
write.csv(seadata50, file = "seadata50", row.names = FALSE)
write.csv(seadata60, file = "seadata60", row.names = FALSE)
write.csv(seadata70, file = "seadata70", row.names = FALSE)
write.csv(seadata80, file = "seadata80", row.names = FALSE)
write.csv(seadata90, file = "seadata90", row.names = FALSE)
write.csv(seadata100, file = "seadata100", row.names = FALSE)
write.csv(seadata110, file = "seadata110", row.names = FALSE)
write.csv(seadata120, file = "seadata120", row.names = FALSE)
write.csv(seadata130, file = "seadata130", row.names = FALSE)
write.csv(seadata140, file = "seadata140", row.names = FALSE)
write.csv(seadata150, file = "seadata150", row.names = FALSE)
write.csv(seadata160, file = "seadata160", row.names = FALSE)
write.csv(seadata170, file = "seadata170", row.names = FALSE)
write.csv(seadata180, file = "seadata180", row.names = FALSE)
write.csv(seadatatot, file = "seadatatot", row.names = FALSE)

###########################
# Sensitivity Analysis
###########################

### Step 1: Get LHS and Simulation Results
nsim <- 300 # Change to value used in LHS
my.sims <- c("seadata10", "seadata20", "seadata30", "seadata40", "seadata50",
             "seadata60", "seadata70", "seadata80", "seadata90", "seadata100",
             "seadata110", "seadata120", "seadata130", "seadata140", 
             "seadata150", "seadata160", "seadata170", "seadata180")
latinframe <- read.csv("latinframe")
seadatatot <- read.csv("seadatatot")

### Step 2: Compute PRCC values for parameters
for( i in 1:18 ){ 
  matrix.name <- paste("prcc",i*10, sep="")
  assign(matrix.name, pcc(latinframe, read.csv(my.sims[i]), nboot = 100,
                          rank = TRUE), envir = .GlobalEnv)
}

### Step 3: Turn PRCC output into usable data frame
my.prcc <- rbind(prcc10$PRCC$original, prcc20$PRCC$original, prcc30$PRCC$original, 
                 prcc40$PRCC$original, prcc50$PRCC$original, prcc60$PRCC$original, 
                 prcc70$PRCC$original, prcc80$PRCC$original, prcc90$PRCC$original,
                 prcc100$PRCC$original, prcc110$PRCC$original, prcc120$PRCC$original,
                 prcc130$PRCC$original, prcc140$PRCC$original, prcc150$PRCC$original, 
                 prcc160$PRCC$original, prcc170$PRCC$original, prcc180$PRCC$original)
for( i in 1:18){
  df.name <- paste("prcc", i * 10, "df", sep = "")
  assign(df.name, my.prcc[i, 1:7])
}

plottime <- seq(10,180,10)
etaovertime <- c(prcc10df[1], prcc20df[1], prcc30df[1], prcc40df[1], prcc50df[1],
                 prcc60df[1], prcc70df[1], prcc80df[1], prcc90df[1], prcc100df[1], 
                 prcc110df[1], prcc120df[1], prcc130df[1], prcc140df[1], 
                 prcc150df[1], prcc160df[1], prcc170df[1], prcc180df[1])
epsilonovertime <- c(prcc10df[2], prcc20df[2], prcc30df[2], prcc40df[2], prcc50df[2],
                     prcc60df[2], prcc70df[2], prcc80df[2], prcc90df[2], prcc100df[2], 
                     prcc110df[2], prcc120df[2], prcc130df[2], prcc140df[2], 
                     prcc150df[2], prcc160df[2], prcc170df[2], prcc180df[2])
iotaovertime <- c(prcc10df[3], prcc20df[3], prcc30df[3], prcc40df[3], prcc50df[3],
                  prcc60df[3], prcc70df[3], prcc80df[3], prcc90df[3], prcc100df[3], 
                  prcc110df[3], prcc120df[3], prcc130df[3], prcc140df[3], 
                  prcc150df[3], prcc160df[3], prcc170df[3], prcc180df[3])
aovertime <- c(prcc10df[4], prcc20df[4], prcc30df[4], prcc40df[4], prcc50df[4], 
               prcc60df[4], prcc70df[4], prcc80df[4], prcc90df[4], prcc100df[4], 
               prcc110df[4], prcc120df[4], prcc130df[4], prcc140df[4], 
               prcc150df[4], prcc160df[4], prcc170df[4], prcc180df[4])
covertime <- c(prcc10df[5], prcc20df[5], prcc30df[5], prcc40df[5], prcc50df[5], 
               prcc60df[5], prcc70df[5], prcc80df[5], prcc90df[5], prcc100df[5], 
               prcc110df[5], prcc120df[5], prcc130df[5], prcc140df[5],
               prcc150df[5], prcc160df[5], prcc170df[5], prcc180df[5])
Betapovertime <- c(prcc10df[6], prcc20df[6], prcc30df[6], prcc40df[6], prcc50df[6], 
                   prcc60df[6], prcc70df[6], prcc80df[6], prcc90df[6], prcc100df[6], 
                   prcc110df[6], prcc120df[6], prcc130df[6], prcc140df[6], 
                   prcc150df[6], prcc160df[6], prcc170df[6], prcc180df[6])
Betacovertime <- c(prcc10df[7], prcc20df[7], prcc30df[7], prcc40df[7], prcc50df[7], 
                   prcc60df[7], prcc70df[7], prcc80df[7], prcc90df[7], prcc100df[7], 
                   prcc110df[7], prcc120df[7], prcc130df[7], prcc140df[7], 
                   prcc150df[7], prcc160df[7], prcc170df[7], prcc180df[7])

### Step 4: Plot Sensitivity Analysis
# Panel A
Graph1 <- ggplot(data = as.data.frame(cbind(plottime, aovertime, covertime, Betapovertime, 
                                            Betacovertime)), 
                 aes(x = plottime, y = aovertime)) +
  labs(x = "Time (days)", y = "PRCC", 
       title = "Parameters Related to Seasonal Factors") +
  geom_line(linetype = "solid", aes(x = plottime, y =  aovertime, color = "a")) +
  geom_line(linetype = "solid", aes(x = plottime, y =  covertime, color = "c")) +
  geom_line(linetype = "solid", aes(x = plottime, y =  Betapovertime, color = "betaP")) + 
  geom_line(linetype = "solid", aes(x = plottime, y =  Betacovertime, color = "betaC")) +
  scale_color_discrete(name = "Parameters", 
                       labels = c("a", expression(beta[C]), expression(beta[P]), "c")) +  
  ylim(-1, 1) +
  theme(panel.background = element_rect(fill = color))

# Panel B
Graph2 <- ggplot(data = as.data.frame(cbind(plottime, etaovertime, epsilonovertime, 
                                            iotaovertime)), 
                 aes(x = plottime, y = etaovertime)) +
  labs(x = "Time (days)", y = "PRCC", 
       title = "Parameters Related to Non-Seasonal Factors") +
  geom_line(linetype = "solid", aes(x = plottime, y =  etaovertime, color = "eta")) +
  geom_line(linetype = "solid", aes(x = plottime, y =  epsilonovertime, color = "epsilon")) +
  geom_line(linetype = "solid", aes(x = plottime, y =  iotaovertime, color = "iota")) + 
  scale_color_discrete(name = "Parameters", 
                       labels = c(expression(epsilon), expression(eta), expression(iota))) +  
  ylim(-1, 1) +
  theme(panel.background = element_rect(fill = color))

# Combine Panels and Save Plot
sensitivity.plot <- plot_grid(Graph1, Graph2, nrow = 2, ncol = 1, labels=c("A", "B"))
ggsave(filename = "sensitivity.pdf", plot = sensitivity.plot, width = 9.39, height = 9.67)

